package base

type DataListOrder struct {
	Sql       string `json:"sql"`
	Type      string `json:"type"`
	Direction string `json:"direction"`
}
type DataListGroup struct {
	Sql  string `json:"sql"`
	Type string `json:"type"`
}

type XlsxImportConfig struct {
	Mode               int      `json:"mode"`
	UniqueImportFields []string `json:"uniqueImportFields"`
}

type DataList struct {
	Uuid                 string           `db:"uuid"`
	ResourceUuid         string           `db:"resource_uuid"`        // 所属资源UUID
	Code                 string           `db:"code"`                 // 列表标识
	Name                 string           `db:"name"`                 // 列表名称
	Title                string           `db:"title"`                // 布局标题
	PrimaryKey           string           `db:"primaryKey"`           // 主键标识
	Layout               string           `db:"layout"`               // 主键标识
	IsOperationColumn    bool             `db:"isOperationColumn"`    // 是否显示操作栏,1是,0否
	OperationColumnWidth int              `db:"operationColumnWidth"` // 操作列宽度
	IsPaginate           bool             `db:"isPaginate"`           // 是否分页
	IsTreeData           bool             `db:"isTreeData"`           // 是否使用树形列表
	TreeDataConfig       map[string]any   `db:"treeDataConfig"`       // 树形列表配置
	IsExpandedRow        string           `db:"isExpandedRow"`        // 列表明细控件
	ExpandedRowConfig    map[string]any   `db:"expandedRowConfig"`    // 列表明细控件配置
	IsTreeSider          bool             `db:"isTreeSider"`          // 是否打开侧边栏侧边栏
	TreeSiderConfig      map[string]any   `db:"treeSiderConfig"`      // 侧边栏设置
	IsDynamicRefresh     bool             `db:"isDynamicRefresh"`     // 是否启用动态刷新
	PageSize             int              `db:"pagesize"`             // 分页大小
	IsAuxiliaryPanel     bool             `db:"isAuxiliaryPanel"`     // 是否使用附属面板
	AuxiliaryPanelConfig map[string]any   `db:"auxiliaryPanelConfig"` // 附属面板配置
	SortConfig           []DataListOrder  `db:"sortConfig"`           // 排序设置
	FieldsSort           []string         `db:"fieldsSort"`           // 字段排序
	GroupConfig          []DataListGroup  `db:"groupConfig"`          // 分组设置
	XlsxFieldsSort       []string         `db:"xlsxFieldsSort"`       // 字段排序
	XlsxImportFieldsSort []string         `db:"xlsxImportFieldsSort"` // 字段排序
	XlsxImportConfig     XlsxImportConfig `db:"xlsxImportConfig"`     // 导入设置
	TabsSort             []string         `db:"tabsSort"`             // 标签排序
	OperationsSort       []string         `db:"operationsSort"`       // 按钮排序
	FsmConfig            [][2]any         `db:"fsmConfig"`            // FSM设置
	ApprovalConfig       [][2]any         `db:"approvalConfig"`       // 审批设置
	UniqueImportFields   []string         `db:"uniqueImportFields"`   // 导入字段唯一效验
	Roles                []string         `db:"roles"`                // 列表权限
	IsAuthDB             int              `db:"is_auth_db"`           // 启用权限过滤
	UpdatedAt            string           `db:"updated_at"`
	CreatedAt            string           `db:"created_at"`
}

type GetDataList func(code string) (DataList, bool)
